/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Spline control point class implementation
 *--------------------------------------------------------------------------
 * @histroy: 16.09.2003. Created
 */
#include "Spline.h"

namespace scene
{

INTERFACE_BEGINMAP(CControlPoint)
  INTERFACE_ENTRY(scene::IControlPoint)
  INTERFACE_ENTRY(core::IErrorInfo)
INTERFACE_ENDMAP()

//global
bool  g_bAutoDetail = false;
long  g_nDetail     = 8;
long  g_nMaxDetail  = 16;


CControlPoint::CControlPoint() : CErrorInfo()
{
  m_dwStatus      = STATUS_ENABLED;
  m_type          = SCP_BEZIER;
  m_vControl[0].z = m_vControl[2].z = 0.0f;
  m_vControl[1].z = m_vControl[3].z = 0.0f;
  m_bAutoDetail   = g_bAutoDetail;
  m_nDetail       = g_bAutoDetail ? -1 : g_nDetail;
}


CControlPoint::~CControlPoint()
{
}

ZRESULT CControlPoint::setType(eControlPoint type)
{
  m_type = type;
  if (m_bAutoDetail)
    m_nDetail = -1;//redetalization required;
  return ZRESULT_OK;
}


ZRESULT CControlPoint::getType(eControlPoint& type)
{
  type = m_type;
  return ZRESULT_OK;
}

ZRESULT CControlPoint::setControlPosition(ZPoint3D pt)
{
  m_ptPosition = pt;
  if (m_bAutoDetail)
    m_nDetail = -1;//redetalization required;
  return ZRESULT_OK;
}

ZRESULT CControlPoint::getControlPosition(ZPoint3D& pt)
{
  pt = m_ptPosition;
  return ZRESULT_OK;
}

ZRESULT CControlPoint::setControlVector(eControlVector type, ZVector v)
{
  m_vControl[(long)type] = v;
  if (SCP_BEZIER == m_type)
    m_vControl[(long)type + ((type == SCV_U0 || type == SCV_V0) ? 1 : -1)] = -v;
  if (m_bAutoDetail)
    m_nDetail = -1;//redetalization required;
  return ZRESULT_OK;
}

ZRESULT CControlPoint::getControlVector(eControlVector type, ZVector& v)
{
  if (m_type == scene::SCP_CORNER)
    v.x = v.y = v.z = 0.0f;
  else
    v = m_vControl[(long)type];
  return ZRESULT_OK;
}

ZRESULT CControlPoint::setDetalization(long nDetail, bool bAuto)
{
  if (m_bAutoDetail || !bAuto)
    m_nDetail = nDetail;
  else
    m_nDetail = -1;
  m_bAutoDetail = bAuto;
  return ZRESULT_OK;
}

ZRESULT CControlPoint::getDetalization(long& nDetail, bool* pAuto)
{
  if (pAuto)
    *pAuto = m_bAutoDetail;
  nDetail = m_nDetail;
  return ZRESULT_OK;
}


//----------------------------------------------
//
// class CSharedControlPoint : implements IControlPoint
//
//----------------------------------------------
CSharedControlPoint::CSharedControlPoint()
{
}

CSharedControlPoint::~CSharedControlPoint()
{
}

ZRESULT CSharedControlPoint::setControlPosition(ZPoint3D pt)
{
  return m_pSharedControl ? m_pSharedControl->setControlPosition(pt) : CControlPoint::setControlPosition(pt);
}

ZRESULT CSharedControlPoint::getControlPosition(ZPoint3D& pt)
{
  return m_pSharedControl ? m_pSharedControl->getControlPosition(pt) : CControlPoint::getControlPosition(pt);
}


};//scene

