/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Tools that change control point type
 *--------------------------------------------------------------------------
 * @histroy: 02.10.2003. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "ControlPointType.h"
#include <helpers/undo.h>

namespace core
{
namespace tools
{

INTERFACE_BEGINMAP(CControlPointType)
  INTERFACE_CHAIN(CContextMenuToolEx)
  INTERFACE_ENTRY(core::layout::IMultyToolLayout)
  INTERFACE_ENTRY(core::IStatus)
INTERFACE_ENDMAP()

CControlPointType::CControlPointType() : CContextMenuToolEx(), CStatus()
{
  m_CurrTool = CORNER;
  m_bAuto    = false;
  m_title[CORNER]       = _T("Corner");
  m_title[BEZIERCORNER] = _T("Bezier-corner");
  m_title[BEZIER]       = _T("Bezier");
  m_title[AUTODETAIL]   = _T("Auto-detail");
  m_seqBranch.add(_T("Type"));
}

CControlPointType::~CControlPointType()
{
}


//-------------------------------------------------------
// @name : CControlPointType::getItemKind
// all items are checkable
// @param DWORD& kind                    : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::getItemKind(DWORD& kind) const
{
  kind = LAYOUT_MM_ITEM_CHECKABLE;
  return ZRESULT_OK;
}


ZRESULT CControlPointType::getStatus(DWORD& nBits)
{
  nBits = STATUS_ENABLED;
  if (m_CurrTool == AUTODETAIL)
    nBits |= (m_bAuto ? STATUS_ACTIVE : 0);
  else
  if (m_CurrTool == CORNER)
    nBits |= ((m_type == scene::SCP_CORNER) ? STATUS_ACTIVE : 0);
  else
  if (m_CurrTool == BEZIERCORNER)
    nBits |= ((m_type == scene::SCP_BEZIERCORNER) ? STATUS_ACTIVE : 0);
  else
  if (m_CurrTool == BEZIER)
    nBits |= ((m_type == scene::SCP_BEZIER) ? STATUS_ACTIVE : 0);
  return ZRESULT_OK;
}

ZRESULT CControlPointType::checkStatus(DWORD nBit)
{
  if (nBit == STATUS_ACTIVE)
  {
    if (m_CurrTool == AUTODETAIL)
      return m_bAuto ? ZRESULT_OK : ZRESULT_FALSE;
    else
    if (m_CurrTool == CORNER)
      return (m_type == scene::SCP_CORNER) ? ZRESULT_OK : ZRESULT_FALSE;
    else
    if (m_CurrTool == BEZIERCORNER)
      return (m_type == scene::SCP_BEZIERCORNER) ? ZRESULT_OK : ZRESULT_FALSE;
    else
    if (m_CurrTool == BEZIER)
      return (m_type == scene::SCP_BEZIER) ? ZRESULT_OK : ZRESULT_FALSE;
  }
  return CStatus::checkStatus(nBit);
}


//-------------------------------------------------------
// @name : CControlPointType::queryLayout
// Queries whether tool can be laid in context menu
// @param scene::INode* pNode            : 
// @param core::eLevel level             : 
// @param core::ui::IViewport* pViewport : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::queryLayout(scene::INode* pNode, core::eLevel level, core::ui::IViewport* pViewport)
{
  m_pThis   = NULL;
  m_pSpline = NULL;
  if (NULL == pNode || level != core::LEV_MANIPULATOR)
    return ZRESULT_FALSE;
  if (m_pSpline && m_pThis && m_CurrTool != CORNER)
    return ZRESULT_OK;//all other tools can lay 
  ZPtr<scene::ISplineNode> pSplineNode;
  if (ZRESULT_OK != pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&pSplineNode))
    return ZRESULT_FALSE;
  if (ZRESULT_OK != pSplineNode->getSpline(&m_pSpline))
    return ZRESULT_FALSE;
  m_pSpline->Release();//StdPtr

  m_bAuto = false;
  //
  // navigate to active control point
  //
  scene::ISpline::lpNode ptr = m_pSpline->head();
  scene::ISpline::lpNode ptrNext = NULL, ptrPrev = NULL;
  m_pNode = pSplineNode;
  if (NULL == ptr)
    return ZRESULT_FALSE;
  if (ZRESULT_OK == m_pSpline->isClosed())
  {
    ptrPrev = ptr;
    while (ptrPrev->m_pNext)
      ptrPrev = ptrPrev->m_pNext;
  }
  while (ptr)
  {
    ptrNext = ptr->m_pNext;
    if (NULL == ptrNext && ZRESULT_OK == m_pSpline->isClosed())
      ptrNext = m_pSpline->head();
    if (ptr->m_pIntr->checkStatus(STATUS_ACTIVE))
    {
      //
      // this point
      //
      m_pThis = ptr->m_pIntr;
      m_pPrev = ptrPrev ? ptrPrev->m_pIntr : NULL;
      m_pNext = ptrNext ? ptrNext->m_pIntr : NULL;
      m_pThis->getType(m_type);
      long nDet = 0;
      m_pThis->getDetalization(nDet, &m_bAuto);
      break;
    }
    ptrPrev = ptr;
    ptr = ptr->m_pNext;
  };
  return m_pThis ? ZRESULT_OK : ZRESULT_FALSE;
}



//-------------------------------------------------------
// @name : CControlPointType::apply
// Change type
// @param core::IProcParams* pParams     : 
// @param DWORD* pRetVal                 : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::apply(core::IProcParams* pParams, DWORD* pRetVal)
{
  if (m_pNode && m_pSpline && m_pThis)
  {
    //undo:
    ZPtr<scene::ISplineNode> pSPNode;
    m_pNode->QueryInterface(IID_(scene::ISplineNode), (void**)&pSPNode);
    core::undo::undoSplineOperation(_T("Change point properties"), 
                                    core::undo::UNDO_SPLINE_GEOMETRY,
                                    pParams,
                                    pSPNode);
    //make cnages
    if (m_CurrTool == AUTODETAIL)
    {
      //change detail
      long nDetail = 8;
      m_pThis->getDetalization(nDetail);
      m_bAuto = !m_bAuto;
      m_pThis->setDetalization(m_bAuto ? -1 : nDetail, m_bAuto);
    }
    else
      alterPoint();
    m_pNode->changed(NC_VISUAL | NC_CHILD | NC_GEOMETRY);
    if (pRetVal)
      *pRetVal |= ZRESULT_REDRAW_ALL;
    return ZRESULT_OK;
  }
  return ZRESULT_FALSE;
}


//-------------------------------------------------------
// @name : CControlPointType::alterPoint
// Point alter routine
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::alterPoint()
{
  if (m_CurrTool == CORNER)
    m_pThis->setType(scene::SCP_CORNER);
  else
  if (m_CurrTool == BEZIERCORNER)
  {
    //
    // bezier-corner point
    //
    scene::eControlPoint oldType;
    m_pThis->getType(oldType);
    m_pThis->setType(scene::SCP_BEZIERCORNER);
    if (oldType == scene::SCP_CORNER)
    {
      ZVector vPrev, vNext;
      if (m_pPrev && m_pThis)
      {
        ZPoint3D p1, p2;
        m_pThis->getControlPosition(p1);
        m_pPrev->getControlPosition(p2);
        m_pPrev->getType(oldType);
        if (oldType != scene::SCP_CORNER)
          m_pPrev->getControlVector(scene::SCV_U0, vPrev);
        vPrev = (p2+vPrev-p1)*0.45f;
      }
      if (m_pNext && m_pThis)
      {
        ZPoint3D p1, p2;
        m_pThis->getControlPosition(p1);
        m_pNext->getControlPosition(p2);
        m_pNext->getType(oldType);
        if (oldType != scene::SCP_CORNER)
          m_pNext->getControlVector(scene::SCV_U1, vNext);
        vNext = (p2+vNext-p1)*0.45f;
      }
      m_pThis->setControlVector(scene::SCV_U0, vNext);
      m_pThis->setControlVector(scene::SCV_U1, vPrev);
    }
  }
  else
  if (m_CurrTool == BEZIER)
  {
    //
    // bezier point
    //
    scene::eControlPoint oldType;
    m_pThis->getType(oldType);
    m_pThis->setType(scene::SCP_BEZIER);
    if (oldType == scene::SCP_CORNER)
    {
      ZVector vPrev, vNext;
      if (m_pPrev && m_pThis)
      {
        ZPoint3D p1, p2;
        m_pThis->getControlPosition(p1);
        m_pPrev->getControlPosition(p2);
        m_pPrev->getControlVector(scene::SCV_U0, vPrev);
        vPrev = (p2+vPrev-p1)*0.45f;
      }
      if (m_pNext && m_pThis)
      {
        ZPoint3D p1, p2;
        m_pThis->getControlPosition(p1);
        m_pNext->getControlPosition(p2);
        m_pNext->getControlVector(scene::SCV_U1, vNext);
        vNext = (p2+vNext-p1)*0.45f;
      }
      ZVector v = (vNext-vPrev)*0.5f;
      m_pThis->setControlVector(scene::SCV_U0, v);
    }
    if (oldType == scene::SCP_BEZIERCORNER)
    {
      ZVector vPrev, vNext, v;
      m_pThis->getControlVector(scene::SCV_U1, vPrev);
      m_pThis->getControlVector(scene::SCV_U0, vNext);
      v = (vNext-vPrev)*0.5f;
      m_pThis->setControlVector(scene::SCV_U0, v);
    }
  }
  return ZRESULT_OK;
}

//-------------------------------------------------------
// @name : CControlPointType::getTitle
// Provides tool title
// @param ZString& strTitle              : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::getTitle(ZString& strTitle)
{
  strTitle = m_title[m_CurrTool];
  return ZRESULT_OK;
}

//-------------------------------------------------------
// @name : CControlPointType::setTitle
// sets tool title
// @param ZString strTitle              : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointType::setTitle(ZString strTitle)
{
  m_title[m_CurrTool] = strTitle;
  return ZRESULT_OK;
}

ZRESULT CControlPointType::setCurrentTool(long nItem)
{
  if (nItem > AUTODETAIL)
    return ZRESULT_FALSE;
  m_CurrTool = (eCurrTool)nItem;
  return ZRESULT_OK;
}

ZRESULT CControlPointType::getBranch(ZStringSeq& branch) const
{
  if (m_CurrTool != AUTODETAIL)
    return CContextMenuToolEx::getBranch(branch);
  else
    branch.length(0);
  return ZRESULT_OK;
}


};//tools
};//core



