#ifndef createsplinetools_h_included
#define createsplinetools_h_included
/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x set of Create\Spline tools
 *--------------------------------------------------------------------------
 * @histroy: 17.09.2003. Created
 */

#include <zmodeler.h>
#include <helpers/layout.h>
#include <helpers/splines.h>
#include <core/tools/IInteractiveTool.h>
#include <core/ui/controls/ISpinControl.h>
#include <core/ui/IViewportDrawWrapper.h>
#include "resource.h"
#define PRECISION 100

namespace core
{
  namespace tools
  {
    // @Name        : class CWeldDialog
    // @Purpose     : Commands bar dialog extension for CStripSplineTool
    class CWeldDialog : public core::tools::CChildDialog
    {
    public:
      CWeldDialog() : core::tools::CChildDialog() {}
      virtual ~CWeldDialog(){}
      CHILDDIALOG_IMPLEMENT(CWeldDialog, IDD_AUTOWELD)
      BOOL    OnInitDialog();
    public:
      float  getThreshold();

      ZPtr<core::ui::controls::ISpinControl>  m_pSpin;
    };

    // @Name        : class CWeldModeDialog
    // @Purpose     : Welding dialog extension for message box
    class CWeldModeDialog : public core::tools::CChildDialog
    {
    public:
      CWeldModeDialog() : core::tools::CChildDialog() {m_nChoice = 0;}
      virtual ~CWeldModeDialog(){}
      CHILDDIALOG_IMPLEMENT(CWeldModeDialog, IDD_WELDMODE)
      BOOL    OnInitDialog();
    public:
      bool  continueSpline();
      bool  stopStartSpline();
    private:
      void getResult();
      UINT  m_nChoice;
    };


    //-------------------------------------------------------------------------
    // @Name        : class CStripSplineTool
    // @Purpose     : Implements Create\Spline\Strip tool
    // @Description : Creates Strip spline in the scene
    //-------------------------------------------------------------------------
    class CStripSplineTool :  public CCommandBarToolEx,
                              public IInteractiveTool,
                              public ui::IViewportDrawWrapper
    {
    public:
      INTERFACE_DECLAREMAP(CStripSplineTool)
      CStripSplineTool();
      virtual ~CStripSplineTool();

    public:
      //---------------------------------------
      //    CCommandBarToolEx
      //---------------------------------------
      ZRESULT requires(DWORD& dwRequires) const;
      ZRESULT getGroup(core::layout::tCommandBarTab& group) const;
      ZRESULT apply(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL);
      ZRESULT done(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL) { return ZRESULT_FALSE; }

    public:
      //---------------------------------------
      //    IInteractiveTool
      //---------------------------------------
      INTERACTIVE_TOOL_DELEGATE(CCommandBarToolEx)
      virtual ZRESULT mouseDown(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseMove(core::IProcParams* pProcParams, ZPoint ptOffset, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseUp(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseWheel(core::IProcParams* pProcParams, DWORD nFlags, short zDelta, DWORD& nRetFlags);
      virtual ZRESULT interrupt(core::IProcParams* pProcParams, DWORD& nRetFlags);

    public:
      //---------------------------------------
      //    IViewportDrawWrapper
      //---------------------------------------
      virtual ZRESULT beforeDraw(core::IProcParams* pParams);
      virtual ZRESULT afterDraw(core::IProcParams* pParams);
      virtual ZRESULT afterShow(core::IProcParams* pParams, CDC* pDC);

    private:
      //implementation:
    private:
      //data members
      ZPtr<CWeldDialog>           m_pDlg;
      ZPtr<CWeldModeDialog>       m_pWeldDlg;
      ZPoint3D                    m_ptOrigin, m_ptLastPoint;
      ZPoint                      m_ptCursorFrom, m_ptPrev, m_ptManip, m_ptCursorTo;
      bool                        m_bCanWeld;

      ZPtr<scene::ISpline>        m_pSpline;  //the spline we are creating.
      ZPtr<scene::ISplineNode>    m_pNode;    //the node it belongs to.
      ZPtr<scene::IControlPoint>  m_pLastControl;
    };//class

  
    // @Name        : class CCircleDialog
    // @Purpose     : Commands bar dialog extension for CCircleSplineTool
    class CCircleDialog : public core::tools::CChildDialog
    {
    public:
      CCircleDialog() : core::tools::CChildDialog() {}
      virtual ~CCircleDialog(){}
      CHILDDIALOG_IMPLEMENT(CCircleDialog, IDD_CIRCLESPLINE)
      DECLARE_MESSAGE_MAP()

    public:
      BOOL    OnInitDialog();
      afx_msg void OnSmooth();
      long    getVertices();
      long    getSegments();
      bool    isSmooth();

      ZPtr<core::ui::controls::ISpinControl>  m_pVertices;
      ZPtr<core::ui::controls::ISpinControl>  m_pSegments;
    };
    //-------------------------------------------------------------------------
    // @Name        : class CCircleSplineTool
    // @Purpose     : Implements Create\Spline\Circle tool
    // @Description : Creates Circle spline in the scene
    //-------------------------------------------------------------------------
    class CCircleSplineTool : public CCommandBarToolEx,
                              public IInteractiveTool
    {
    public:
      INTERFACE_DECLAREMAP(CCircleSplineTool)
      CCircleSplineTool();
      virtual ~CCircleSplineTool();

    public:
      //---------------------------------------
      //    CCommandBarToolEx
      //---------------------------------------
      ZRESULT requires(DWORD& dwRequires) const;
      ZRESULT getGroup(core::layout::tCommandBarTab& group) const;
      ZRESULT apply(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL);
      ZRESULT done(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL) { return ZRESULT_FALSE; }

    public:
      //---------------------------------------
      //    IInteractiveTool
      //---------------------------------------
      INTERACTIVE_TOOL_DELEGATE(CCommandBarToolEx)
      virtual ZRESULT mouseDown(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseMove(core::IProcParams* pProcParams, ZPoint ptOffset, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseUp(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseWheel(core::IProcParams* pProcParams, DWORD nFlags, short zDelta, DWORD& nRetFlags);
      virtual ZRESULT interrupt(core::IProcParams* pProcParams, DWORD& nRetFlags) { return ZRESULT_OK; }

    private:
      //implementation:
    private:
      //data members
      ZPtr<CCircleDialog>         m_pDlg;
      ZPoint3D                    m_ptOrigin, m_ptLastPoint;
      ZPoint                      m_ptCursorFrom, m_ptCursorTo;
      bool                        m_bCircle;
    };//class


    // @Name        : class CHelixDialog
    // @Purpose     : Commands bar dialog extension for CHelixSplineTool
    class CHelixDialog : public core::tools::CChildDialog
    {
    public:
      CHelixDialog() : core::tools::CChildDialog() {}
      virtual ~CHelixDialog(){}
      CHILDDIALOG_IMPLEMENT(CHelixDialog, IDD_HELIXSPLINE)
      DECLARE_MESSAGE_MAP()

    public:
      BOOL    OnInitDialog();
      afx_msg void OnSmooth();
      long    getVertices();
      long    getSegments();
      long    getSteps();
      bool    isSmooth();

      ZPtr<core::ui::controls::ISpinControl>  m_pVertices;
      ZPtr<core::ui::controls::ISpinControl>  m_pSegments;
      ZPtr<core::ui::controls::ISpinControl>  m_pSteps;
    };
    //-------------------------------------------------------------------------
    // @Name        : class CHelixSplineTool
    // @Purpose     : Implements Create\Spline\Helix tool
    // @Description : Creates Circle spline in the scene
    //-------------------------------------------------------------------------
    class CHelixSplineTool :  public CCommandBarToolEx,
                              public IInteractiveTool
    {
    public:
      INTERFACE_DECLAREMAP(CHelixSplineTool)
      CHelixSplineTool();
      virtual ~CHelixSplineTool();

    public:
      //---------------------------------------
      //    CCommandBarToolEx
      //---------------------------------------
      ZRESULT requires(DWORD& dwRequires) const;
      ZRESULT getGroup(core::layout::tCommandBarTab& group) const;
      ZRESULT apply(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL);
      ZRESULT done(core::IProcParams* pParams = NULL, DWORD* pRetVal = NULL) { return ZRESULT_FALSE; }

    public:
      //---------------------------------------
      //    IInteractiveTool
      //---------------------------------------
      INTERACTIVE_TOOL_DELEGATE(CCommandBarToolEx)
      virtual ZRESULT mouseDown(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseMove(core::IProcParams* pProcParams, ZPoint ptOffset, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseUp(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags);
      virtual ZRESULT mouseWheel(core::IProcParams* pProcParams, DWORD nFlags, short zDelta, DWORD& nRetFlags);
      virtual ZRESULT interrupt(core::IProcParams* pProcParams, DWORD& nRetFlags) { return ZRESULT_OK; }

    private:
      //implementation:
    private:
      //data members
      ZPtr<CHelixDialog>          m_pDlg;
      ZPoint3D                    m_ptOrigin, m_ptLastPoint;
      ZPoint                      m_ptCursorFrom, m_ptCursorTo;
      bool                        m_bCircle;
    };//class
  };//tools
};//core

#endif //createsplinetools_h_included

