/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2003 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x Create\Light\Strip tool implementation
 *--------------------------------------------------------------------------
 * @histroy: 17.09.2003. Created
 */
#include "Spline.h"
#include "SplineNode.h"
#include "CreateSpline.h"
#include <zmsharedclasses.h>
#include <helpers/events.h>
#include <helpers/services.h>
#include <helpers/viewportDC.h>
#include <helpers/sceneNavigation.h>
#include <helpers/splines.h>
#include <core/ui/ISceneViewport.h>
#include <scene/ITransformable.h>
#include <helpers/undo.h>

namespace core
{
namespace tools
{

INTERFACE_BEGINMAP(CStripSplineTool)
  INTERFACE_CHAIN(CCommandBarToolEx)
  INTERFACE_ENTRY(core::tools::IInteractiveTool)
  INTERFACE_ENTRY(core::ui::IViewportDrawWrapper)
INTERFACE_ENDMAP()

CStripSplineTool::CStripSplineTool() : CCommandBarToolEx(core::ui::CURSOR_CREATE)
{
  m_strName = m_strTitle = _T("Strip");
  m_seqBranch.add(_T("Spline..."));
  m_seqDesc.add(_T("Creates a spline."));
  m_seqDesc.add(_T("Click (and drag) to create the spline. Right-click when done. Expand options box for additional settings."));
  CWeldDialog::CreateInstance(&m_pDlg);
  m_pDlg->setDialogTitle(_T("Options"));
  m_pDialogs->addLast(m_pDlg);
  CWeldModeDialog::CreateInstance(&m_pWeldDlg);
  m_bCanWeld = true;
}

CStripSplineTool::~CStripSplineTool()
{
}


//-------------------------------------------------------
// @name : CStripSplineTool::requires
// Tool requirements
// @param DWORD& dwRequires              : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::requires(DWORD& dwRequires) const
{
  dwRequires = TOOL_REQUIRES_MOUSEL | TOOL_REQUIRES_MOUSER | TOOL_REQUIRES_LEV_NODE;
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CStripSplineTool::getGroup
// Tool qualifying branch
// @param core::layout::tCommandBarTab& group : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::getGroup(core::layout::tCommandBarTab& group) const
{
  group = core::layout::LAYOUT_CB_TAB_CREATE;
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CStripSplineTool::apply
// The application routine
// @param core::IProcParams* pParams     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::apply(core::IProcParams* pParams, DWORD* pRetVal)
{
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CStripSplineTool::mouseDown
// A-la starting creation
// @param core::IProcParams* pProcParams : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::mouseDown(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags)
{
  resetError();
  if ((0 == (nFlags & (MK_LBUTTON | MK_RBUTTON))) || NULL == pProcParams)
    return ZRESULT_FALSE;
  //
  // interrupt condition
  //
  if (nFlags & MK_RBUTTON)
  {
    if (m_pSpline)
      m_pSpline->erase(m_pLastControl);
    if (m_pNode)
      m_pNode->changed(NC_VISUAL | NC_GEOMETRY);
    nRetFlags |= ZRESULT_REDRAW_ALL;
    return interrupt(pProcParams, nRetFlags);
  }

  pProcParams->getCursorLocation(m_ptCursorFrom);
  m_ptCursorTo = m_ptCursorFrom;
  ZLine3D line;
  pProcParams->getInputLine(line);

  //
  // start new spline?
  //
  if (!m_pSpline)
  {
    m_pNode = NULL;
    m_pLastControl = NULL;
    bool bNew  = true;
    bool bHead = false;
    bool bMatch = ZRESULT_OK == scene::findSplineWeldVertex(
                                      pProcParams, m_pDlg->getThreshold(), m_pLastControl,
                                      &m_pSpline, &m_pNode, &bHead);
    if (bMatch)
    {
      messageBoxDlg(m_pWeldDlg, _T("What action would you like to perform on the spline you've clicked?"),
                                _T("Spline creation mode:"),
                                MB_ICONQUESTION | MB_OK);
      bNew = m_pWeldDlg->stopStartSpline();
    }
    if (bNew)
    {
      m_pSpline = NULL;
      m_pNode = NULL;
      //create new
      scene::createSplineNode(&m_pNode);
      scene::CSpline::CreateInstance((void**)&m_pSpline);
      m_pNode->setSpline(m_pSpline);
      ZPtr<scene::ITransformable> pTrans;
      if (ZRESULT_OK == m_pNode->QueryInterface(IID_(scene::ITransformable), (void**)&pTrans))
      {
        ZMatrix mat;
        m_ptOrigin = line[1];
        mat.setTranslation(m_ptOrigin);
        pTrans->setTransform(mat);
      }

      //create common undo
      core::undo::undoNodeOperation(_T("Create SplineStrip"), core::undo::UNDO_ADD_NODE, pProcParams, m_pNode);
      //
      // add node to the scene:
      //
      ZPtr<scene::IScene>         pScene;
      if (ZFAILED(g_pZModeler->getScene(&pScene)))
        return ZRESULT_FAIL;
      pScene->attachNode(m_pNode, NULL);

      //
      // create spline control point
      //
      ZPoint3D pt = line[1];
      ZPtr<scene::IControlPoint>  pControl;
      scene::CControlPoint::CreateInstance((void**)&pControl);
      pControl->setControlPosition(pt-m_ptOrigin);
      ZVector vZero;
      pControl->setType(scene::SCP_CORNER);
      pControl->setControlVector(scene::SCV_U0, vZero);
      pControl->setControlVector(scene::SCV_U1, vZero);
      m_pLastControl = pControl;
      m_pSpline->addLast(pControl);
    }
    else
    {
      //continue existen
      //get spline origin
      ZPtr<scene::ITransformable> pTrans;
      if (ZRESULT_OK == m_pNode->QueryInterface(IID_(scene::ITransformable), (void**)&pTrans))
      {
        ZMatrix mat;
        pTrans->getTransform(mat);
        mat.getTranslation(m_ptOrigin);
      }
    }

    if (!bNew)
    {
      //continue existen spline
      m_pLastControl = NULL;
      if (bHead)
        m_pSpline->reverseOrder();
      m_pSpline->revreset();
      m_pSpline->prev(&m_pLastControl);
    }
    m_pNode->changed(NC_VISUAL | NC_GEOMETRY | NC_TRANSFORM);
    //
    // if bMatch (there was a dialog box: we've missed a mouseUp event)
    //
    nRetFlags |= ZRESULT_REDRAW_ALL;
    if (bMatch)
    {
      m_bCanWeld = false;
      mouseUp(pProcParams, nFlags | MK_LBUTTON, nRetFlags);//was missed
      m_bCanWeld = true;
    }
  }//start new spline
  m_ptLastPoint = line[1];
  if (!m_pSpline)
    return REPORT_ERROR(NULL, _T("Failed to create new spline"));
  return ZRESULT_OK;//captured event
}


//-------------------------------------------------------
// @name : CStripSplineTool::mouseMove
// Dragging the mouse
// @param core::IProcParams* pProcParams : 
// @param ZPoint ptOffset                : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::mouseMove(core::IProcParams* pProcParams, ZPoint ptOffset, DWORD nFlags, DWORD& nRetFlags)
{
  if (NULL == pProcParams || !m_pLastControl || !m_pNode)
    return ZRESULT_FALSE;
  ZLine3D line;
  pProcParams->getInputLine(line);
  if (nFlags & MK_LBUTTON)
  { 
    // dragging manipulator
    ZVector v = line[1]-m_ptLastPoint;
    m_pLastControl->setType(v.length() > m_pDlg->getThreshold() ? scene::SCP_BEZIER : scene::SCP_CORNER);
    m_pLastControl->setControlVector(scene::SCV_U0, v);
    ZPtr<core::ui::IViewport> pVP;
    pProcParams->getViewport(&pVP);
    if (pVP)
      pVP->addWrapper(this, true);
  }
  else
    m_pLastControl->setControlPosition(line[1]-m_ptOrigin);

  if (m_pSpline->length() >= 2)
  {
    //alter detalization on previouse (to m_pLastControl) point
    ZPtr<scene::IControlPoint>  pt;
    m_pSpline->reset();
    m_pSpline->skip(m_pSpline->length()-2);
    if (ZRESULT_OK == m_pSpline->next(&pt))
    {
      bool bAuto;
      long nDet = 0;
      pt->getDetalization(nDet, &bAuto);
      if (bAuto)
        pt->setDetalization(-1, true);
    }
  }//alter detalization

  nRetFlags |= ZRESULT_REDRAW_ALL;
  m_pNode->changed(NC_VISUAL | NC_GEOMETRY);
  return ZRESULT_OK;//captured event
}


//-------------------------------------------------------
// @name : CStripSplineTool::mouseUp
// Create the light!
// @param core::IProcParams* pProcParams : 
// @param DWORD nFlags                   : 
// @param DWORD& nRetFlags               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::mouseUp(core::IProcParams* pProcParams, DWORD nFlags, DWORD& nRetFlags)
{
  if ((0 == (nFlags & MK_LBUTTON)) ||
    NULL == pProcParams || !m_pSpline || !m_pNode || !m_pLastControl)
    return ZRESULT_FALSE;

  if (m_bCanWeld)
  {
    ZPtr<scene::ISplineNode>  pWeldNode;
    ZPtr<scene::ISpline>      pWeldSpline;
    bool bContinue = true;
    bool bHead = false;
    bool bMatch = ZRESULT_OK == scene::findSplineWeldVertex(
                                    pProcParams, m_pDlg->getThreshold(), m_pLastControl,
                                    &pWeldSpline, &pWeldNode, &bHead);
    if (bMatch)
    {
      messageBoxDlg(m_pWeldDlg, _T("What action would you like to perform on the spline you've clicked?"),
                                _T("Spline creation mode:"),
                                MB_ICONQUESTION | MB_OK);
      bContinue = m_pWeldDlg->continueSpline();
    }
    if (!bContinue && (pWeldSpline == m_pSpline))
    {
      m_pSpline->erase(m_pLastControl);//erase since it matches our spline's head
      m_pNode->changed(NC_VISUAL | NC_GEOMETRY);//allow node to change data
      m_pLastControl = NULL;
      pWeldSpline->reset();
      pWeldSpline->next(&m_pLastControl);
      scene::eControlPoint type;
      m_pLastControl->getType(type);
      if (type != scene::SCP_CORNER)
        m_pLastControl->setType(scene::SCP_BEZIERCORNER);
      ZVector vZero;
      m_pLastControl->setControlVector(scene::SCV_U1, vZero);
      pWeldSpline->setClosed(true);
      m_pNode->changed(NC_VISUAL | NC_GEOMETRY);
      nRetFlags |= ZRESULT_REDRAW_ALL;
      interrupt(pProcParams, nRetFlags);
      return ZRESULT_OK;
    }
  }
  //
  // create spline control point
  //
  ZLine3D line;
  pProcParams->getInputLine(line);
  ZPoint3D pt = line[1];
  ZPtr<scene::IControlPoint>  pControl;
  scene::CControlPoint::CreateInstance((void**)&pControl);
  pControl->setControlPosition(pt-m_ptOrigin);
  ZVector vZero;
  pControl->setControlVector(scene::SCV_U0, vZero);
  pControl->setControlVector(scene::SCV_U1, vZero);
  pControl->setType(scene::SCP_CORNER);
  m_pSpline->addLast(pControl);

  m_pNode->changed(NC_VISUAL | NC_GEOMETRY);
  m_ptLastPoint = pt;
  m_pLastControl = pControl;

  nRetFlags |= ZRESULT_REDRAW_ALL;
  return ZRESULT_OK;
}

ZRESULT CStripSplineTool::mouseWheel(core::IProcParams* pProcParams, DWORD nFlags, short zDelta, DWORD& nRetFlags)
{
  return ZRESULT_FALSE;//not our case
}

//
// IViewportDrawWrapper
//

//-------------------------------------------------------
// @name : CStripSplineTool::afterShow
// After the viewport content have been shown
// @param CDC* pDC                       : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::afterShow(core::IProcParams* pParams, CDC* pDC)
{
  if (0 == (0x80 & GetKeyState(VK_LBUTTON)))
    return ZRESULT_FALSE;
  if (NULL == pDC)
    return ZRESULT_FALSE;
  CPen pen;
  pen.CreatePen(PS_DOT, 1, RGB(0, 0, 255));
  pDC->SelectStockObject(NULL_BRUSH);
  CPen* pOldPen = pDC->SelectObject(&pen);
  pParams->getCursorLocation(m_ptCursorTo);
  pDC->MoveTo(m_ptCursorFrom.x(), m_ptCursorFrom.y());
  pDC->LineTo(m_ptCursorTo.x(), m_ptCursorTo.y());
  pDC->SelectObject(pOldPen);
  return ZRESULT_FALSE;
}

//-------------------------------------------------------
// @name : CStripSplineTool::afterDraw
// Draws dashed line while creating a spline
// @param core::IProcParams* pParams     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::afterDraw(core::IProcParams* pParams)
{
  return ZRESULT_FALSE;
}

ZRESULT CStripSplineTool::beforeDraw(core::IProcParams* pParams)
{
  return ZRESULT_FALSE;
}

//-------------------------------------------------------
// @name : CStripSplineTool::interrupt
// Comment  
// @param core::IProcParams* pProcParams : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CStripSplineTool::interrupt(core::IProcParams* pProcParams, DWORD& nRetFlags)
{
  m_pSpline       = NULL;
  m_pNode         = NULL;
  m_pLastControl  = NULL;
  return ZRESULT_OK;
}


};//tools
};//core

