/***************************************************************************
 *                          Zanoza Modeler v2.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                   Copyright (c) 2002 Zanoza Software.                   *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  ZModeler 2.x general properties implementation
 *--------------------------------------------------------------------------
 * @histroy: 29.10.2003. Created
 */
#include "controlPointProperties.h"
#include <scene/ISplineNode.h>
#include <helpers/services.h>
#include <helpers/nodes.h>
#include <helpers/undo.h>
#include <helpers/sceneNavigation.h>
#include <zmsharedclasses.h>

namespace core
{
namespace tools
{

INTERFACE_BEGINMAP(CControlPointProperties)
  INTERFACE_ENTRY(services::IPropertiesSource)
INTERFACE_ENDMAP()
  
CControlPointProperties::CControlPointProperties()
{
  m_seqTypeNames[scene::SCP_BEZIERCORNER] = _T("Bezier Corner");
  m_seqTypeNames[scene::SCP_BEZIER]       = _T("Bezier");
  m_seqTypeNames[scene::SCP_CORNER]       = _T("Corner");
  //other
}

CControlPointProperties::~CControlPointProperties()
{
}

//-----------------------
// IPropertiesSource
//-----------------------
ZRESULT CControlPointProperties::applyProperties(core::IProcParams* pParams, 
                                            core::eLevel level, 
                                            scene::tNodeSeq& seqNodes, 
                                            services::IPropertiesService* pService)
{
  ZPtr<core::ICompatibilityInfo>  pCompInfo;
  g_pZModeler->getCurrentCompatibility(&pCompInfo);

  int i;
  DWORD dwExBitsAdd = 0, dwExBitsRem = 0;
  ZStringSeq seq;
  seq.add(_T("General"));
  seq.add(_T("External State"));
  //ID, UV, etc:
  bool bChangeID = ZRESULT_OK == pService->isPropertyChanged(seq, _T("ID"));
  bool bChangeUV = ZRESULT_OK == pService->isPropertyChanged(seq, _T("UV/User float"));
  bool bChangeDetail = ZRESULT_OK == pService->isPropertyChanged(seq, _T("Detail"));
  bool bChangeAutoDetail = ZRESULT_OK == pService->isPropertyChanged(seq, _T("Auto-detail"));
  bool bChangeType = ZRESULT_OK == pService->isPropertyChanged(seq, _T("Type"));
  //compatibility info:
  ZPtr<core::INamed> pNamed;
  pCompInfo->QueryInterface(IID_(core::INamed), (void**)&pNamed);
  ZString strCompName;
  pNamed->getTitle(strCompName);
  seq.add(strCompName);
  for (i = 0; i < 8; i++)
    if (m_seqBitsNames[i].length() &&
        (ZRESULT_OK == pService->isPropertyChanged(seq, m_seqBitsNames[i])))
      if (m_bCompBits[i])
        dwExBitsAdd |= (1<<i);
      else
        dwExBitsRem |= (1<<i);
  seq.length(seq.length()-2);

  for (i = 0; i < seqNodes.length(); i++)
  {
    ZPtr<scene::ISplineNode>  pSplineNode;
    ZPtr<scene::ISpline>      pSpline;
    if (ZRESULT_OK == seqNodes[i]->QueryInterface(IID_(scene::ISplineNode), (void**)&pSplineNode) &&
        ZRESULT_OK == pSplineNode->getSpline(&pSpline))
    {
      //
      // undo spline stuff
      //
      core::undo::undoSplineOperation(_T("Spline changes"), 
                                      core::undo::UNDO_SPLINE_ALL, 
                                      pParams, 
                                      pSplineNode);
      //alter control points
      ZPtr<scene::IControlPoint>  ptr;
      pSpline->reset();
      while (ZRESULT_OK == pSpline->next(&ptr))
      {
        if (!ptr->checkStatus(STATUS_SELECTED))
        {
          ptr = NULL;
          continue;
        }
        // ID:
        if (bChangeID)
          ptr->ID() = (WORD)m_nID;
        // UV:
        if (bChangeUV)
          ptr->userFloat() = m_fUV;
        // detail
        if (bChangeDetail)
          ptr->setDetalization(m_nDetail, bChangeAutoDetail ? m_bAuto : false);
        else
          if (bChangeAutoDetail)
            ptr->setDetalization(0, m_bAuto);// -1 or 0?
        // type
        if (bChangeType && m_nType >=0 && m_nType < m_seqTypeNames.length())
          ptr->setType((scene::eControlPoint)m_nType);
        // compatibility bits
        if (dwExBitsAdd | dwExBitsRem)
          ptr->compatibility() = (BYTE)((ptr->compatibility() & ~dwExBitsRem) | dwExBitsAdd);
        ptr = NULL;
      }//while points
    }//spline
  }//loop on nodes
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CControlPointProperties::queryProperties
// Queries for general properties
// @param scene::tNodeSeq& seqNodes      : 
// @param services::IPropertiesService* pService : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CControlPointProperties::queryProperties(core::IProcParams* pParams, 
                                            core::eLevel level, 
                                            scene::tNodeSeq& seqNodes, 
                                            services::IPropertiesService* pService)
{
  if (level != LEV_MANIPULATOR)
    return ZRESULT_FALSE;
  ZStringSeq seq;
  seq.add(_T("General"));
  long  nIDInfo         = 0; //0 - unassigned, > 0 assigned, < 0 different
  long  nUVInfo         = 0; //0 - unassigned, > 0 assigned, < 0 different
  long  nDetailInfo     = 0; //0 - unassigned, > 0 assigned, < 0 different
  long  nAutoDetailInfo = 0; //0 - unassigned, > 0 assigned, < 0 different
  long  nTypeInfo       = 0; //0 - unassigned, > 0 assigned, < 0 different
  m_nID                 = 0;
  m_fUV                 = 0.0f;
  m_nDetail             = 0;
  m_bAuto               = false;
  m_nType               = scene::SCP_CORNER;
  ZPtr<core::ICompatibilityInfo>  pCompInfo;
  g_pZModeler->getCurrentCompatibility(&pCompInfo);
  if (pCompInfo)
    pCompInfo->getPolyBitsNames(m_seqBitsNames);
  long  nBitsInfo[8]    = {0, 0, 0, 0, 0, 0, 0, 0};

  for (int i = 0; i < seqNodes.length(); i++)
  {
    ZPtr<scene::ISplineNode>  pSplineNode;
    ZPtr<scene::ISpline>      pSpline;
    if (ZRESULT_OK == seqNodes[i]->QueryInterface(IID_(scene::ISplineNode), (void**)&pSplineNode) &&
        ZRESULT_OK == pSplineNode->getSpline(&pSpline))
    {
      ZPtr<scene::IControlPoint>  ptr;
      pSpline->reset();
      while (ZRESULT_OK == pSpline->next(&ptr))
      {
        if (!ptr->checkStatus(STATUS_SELECTED))
        {
          ptr = NULL;
          continue;
        }
        // ID:
        if (0 == nIDInfo)
        {
          m_nID = ptr->ID();
          nIDInfo = 1;
        }
        else
          if (nIDInfo > 0 && m_nID != ptr->ID())
            nIDInfo = -1;
        // UV:
        if (0 == nUVInfo)
        {
          m_fUV = ptr->userFloat();
          nUVInfo = 1;
        }
        else
          if (nUVInfo > 0 && m_fUV != ptr->userFloat())
            nUVInfo = -1;
        // Detail:
        long nDetail;
        bool bAuto;
        ptr->getDetalization(nDetail, &bAuto);
        if (0 == nDetailInfo)
        {
          m_nDetail = nDetail;
          nDetailInfo = 1;
        }
        else
          if (nDetailInfo > 0 && m_nDetail != nDetail)
            nDetailInfo = -1;
        if (0 == nAutoDetailInfo)
        {
          m_bAuto = bAuto;
          nAutoDetailInfo = 1;
        }
        else
          if (nAutoDetailInfo > 0 && m_bAuto != bAuto)
            nAutoDetailInfo = -1;
        //type
        scene::eControlPoint type;
        ptr->getType(type);
        if (0 == type)
        {
          m_nType = (long)type;
          nTypeInfo = 1;
        }
        else
          if (nTypeInfo > 0 && m_nType != (long)type)
            nTypeInfo = -1;
        // compatibility bits
        for (int b = 0; b < 8; b++)
          if (m_seqBitsNames[b].length())
          {
            if (nBitsInfo[b] == 0)
            {
              nBitsInfo[b] = 1;
              m_bCompBits[b] = (0!=(ptr->compatibility() & (1<<b)));
            }
            else
              if (nBitsInfo[b] > 0 && m_bCompBits[b] != (0!=(ptr->compatibility() & (1<<b))))
                nBitsInfo[b] = -1;
          }

        ptr = NULL;
      }//control points
    }//spline
  }//loop on nodes
  //ID:
  seq.add(_T("External State"));
  if (nIDInfo != 0)
    pService->addLongRangeProperty(seq, _T("ID"), &m_nID, 0, 0x7FFF, nIDInfo > 0);
  if (nUVInfo != 0)
    pService->addFloatProperty(seq, _T("UV/User float"), &m_fUV, nUVInfo > 0);
  if (nAutoDetailInfo != 0)
    pService->addBoolProperty(seq, _T("Auto-detail"), &m_bAuto, nAutoDetailInfo > 0);
  if (nDetailInfo != 0)
    pService->addLongRangeProperty(seq, _T("Detail"), &m_nDetail, 0, 0x100, nDetailInfo > 0);
  if (nTypeInfo != 0)
    pService->addVariationsProperty(seq, _T("Type"), m_seqTypeNames, &m_nType, nTypeInfo > 0);
  //compatibility info:
  ZPtr<core::INamed> pNamed;
  pCompInfo->QueryInterface(IID_(core::INamed), (void**)&pNamed);
  ZString strCompName;
  pNamed->getTitle(strCompName);
  seq.add(strCompName);
  for (int b = 0; b < 8; b++)
    if (nBitsInfo[b] != 0)
      pService->addBoolProperty(seq, m_seqBitsNames[b], &m_bCompBits[b], nBitsInfo[b] > 0);
  seq.length(seq.length()-2);

  return ZRESULT_OK;
}


};//tools
};//core

